﻿using CommunityToolkit.Mvvm.Input;
using RenderableSeriesViewModel.Models;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.Axes;
using SciChart.Core.Extensions;
using SciChart.Data.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace RenderableSeriesViewModel.ViewModels
{
    public partial class MainViewModel : SharedViewModel
    {
        public MainViewModel()
        {
            FilePath = "Resources/Data/test.csv";
        }

        // define rangeYAxis as a static variable to be used in the custom zoom extent
        public static ObservableCollection<NumericAxisViewModel> _rangeYAxis;

        public static ObservableCollection<NumericAxisViewModel> RangeYAxis
        {
            get { return _rangeYAxis; }
            set
            {
                _rangeYAxis = value;
            }
        }

        [RelayCommand]
        private void LoadFile()
        {
            if (!string.IsNullOrEmpty(FilePath))
            {
                ConvertFileToSeries();
            }
        }

        private async void ConvertFileToSeries()
        {
            try
            {
                // read csv file and convert to list of TaggedData
                await Task.Run(() => CSVtoList());

                if (Series.Count == 0)
                {
                    MessageBox.Show("No data found in file", "ERROR", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                else
                {
                    // sort list by tag
                    Series = Series.OrderBy(item => item.Tag).ToList();
                    ProcessSeries();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }

        private void CSVtoList()
        {
            try
            {
                Series = new List<TaggedData>();
                var lines = File.ReadLines(FilePath, Encoding.UTF8).ToList();
                var columnNames = lines[0].Split(new[] { ',', ';' });

                // Read first line and initialize list of tags
                foreach (var (tag, index) in columnNames.Skip(1).Select((value, index) => (value, index)))
                {
                    //Créate new tag if tag is empty
                    var tagName = string.IsNullOrEmpty(tag) ? $"UnknownTag{index + 1}" : tag.Trim();
                    //add tag to list
                    Series.Add(new TaggedData { Tag = tagName, Points = new XyDataSeries<DateTime, double>() });
                }

                foreach (var line in lines.Skip(1))
                {
                    //read next line and split it
                    var cells = line.Split(new[] { ',', ';' });

                    // Parse date and check if it is valid
                    if (!DateTime.TryParse(cells[0], CultureInfo.GetCultureInfo("fr-FR"), DateTimeStyles.None, out DateTime date))
                    {
                        if (!DateTime.TryParse(cells[0], CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                        {
                            MessageBox.Show("Date format error", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                            return;
                        }
                    }

                    // Parse and add value if possible
                    for (int i = 1; i <= cells.Length - 1; i++)
                    {
                        // parse value and add it to the tag if not NaN and date not minvalue
                        if (double.TryParse(cells[i], out double value) && !double.IsNaN(value) && date != DateTime.MinValue)
                        {
                            Series[i - 1].Points.Append(date, value);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error reading file", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void ProcessSeries()
        {
            GenerateLineChart();

            // get the min and max of the x axis
            DateTime xMin = Series.Where(item => item.Points.Count > 0).Select(item => item.Points.XRange.Min).Min().ToDateTime();
            DateTime xMax = Series.Where(item => item.Points.Count > 0).Select(item => item.Points.XRange.Max).Max().ToDateTime();

            // set the x axis visible range on new file load
            XVisibleRange = new DateRange(xMin, xMax);

            // Creating a deep copy of the original ObservableCollection to be used for custom zoom extent.
            RangeYAxis = new ObservableCollection<NumericAxisViewModel>(YAxes.Select(item => new NumericAxisViewModel { VisibleRange = item.VisibleRange, Id = item.Id }));


            ChartTitle = "File Name: " + Path.GetFileName(FilePath);

        }

        private void GenerateLineChart()
        {
            var lst = new ObservableCollection<Color>();
            Color color = (Color)ColorConverter.ConvertFromString("#008000");
            lst.Add(color);
            color = (Color)ColorConverter.ConvertFromString("#ffff00");
            lst.Add(color);
            color = (Color)ColorConverter.ConvertFromString("#0000ff");
            lst.Add(color);
            color = (Color)ColorConverter.ConvertFromString("#ff0000");
            lst.Add(color);

            var seriesColors = new Stack<Color>(lst);

            RenderableSeriesViewModels = new ObservableCollection<LineRenderableSeriesViewModel>();
            YAxes = new ObservableCollection<IAxisViewModel>();

            foreach (var serie in Series)
            {
                // declare new data series
                XyDataSeries<DateTime, double>? dataSeries = serie.Points;
                dataSeries.SeriesName = serie.Tag;

                // add data to series
                if (dataSeries.Count > 0)
                {
                    // define visible range
                    IRange visibleRangeXY = new DoubleRange(-1, 1);

                    // define axis color
                    var axisColor = seriesColors.Pop();

                    // define axis

                    YAxes.Add(new NumericAxisViewModel()
                    {
                        AxisTitle = serie.Tag,
                        Id = serie.Tag,
                        AutoRange = AutoRange.Once,
                        IsPrimaryAxis = false,
                        Visibility = Visibility.Collapsed, //Collapsed
                        AxisAlignment = AxisAlignment.Right,
                        VisibleRange = visibleRangeXY,
                        StrokeThickness = 2,
                        BorderThickness = new Thickness(2, 0, 0, 0),
                        BorderBrush = new SolidColorBrush(axisColor),
                        TickTextBrush = new SolidColorBrush(axisColor),
                        DrawMajorTicks = true,
                        DrawMinorTicks = true,
                    });

                    // define series
                    var lineSeries = new LineRenderableSeriesViewModel()
                    {
                        DataSeries = dataSeries,
                        Tag = serie.Tag,
                        Stroke = axisColor,
                        StrokeThickness = 1,
                        AntiAliasing = true,
                        IsDigitalLine = IsDigitalLine,
                        YAxisId = serie.Tag,
                        StyleKey = "ChartTooltipStyle",
                        IsVisible = true,
                    };

                    // add series to list
                    RenderableSeriesViewModels.Add(lineSeries);
                }
            }
        }

    }
}
